@@ -18,9 +18,6 @@
 **
 ****************************************************************************/
 
-
-
-
 #include "GraphDraw/maingraph.h"
 
 
@@ -39,65 +36,44 @@ MainGraph::MainGraph(Information *info) : GraphDraw(info)
     cancelUpdateSignal = false;
     resaveTangent = animationUpdate = false;
 
-    mouseNotOnHWidget.setInterval(1000);
-    mouseNotOnHWidget.setSingleShot(true);
-
-    connect(&mouseNotOnHWidget, SIGNAL(timeout()), &hWidgetHideTransition, SLOT(start()));
-
-    mouseNotOnVWidget.setInterval(1000);
-    mouseNotOnVWidget.setSingleShot(true);
-
-    connect(&mouseNotOnVWidget, SIGNAL(timeout()), &vWidgetHideTransition, SLOT(start()));
-
-    hWidgetHideTransition.setInterval(30);
-    vWidgetHideTransition.setInterval(30);
-
-    hWidgetShowTransition.setInterval(30);
-    vWidgetShowTransition.setInterval(30);
-
-    connect(&hWidgetHideTransition, SIGNAL(timeout()), this, SLOT(hideHorWidget()));
-    connect(&vWidgetHideTransition, SIGNAL(timeout()), this, SLOT(hideVerWidget()));
-    connect(&hWidgetShowTransition, SIGNAL(timeout()), this, SLOT(showHorWidget()));
-    connect(&vWidgetShowTransition, SIGNAL(timeout()), this, SLOT(showVerWidget()));
-
-    timeWaitForXYWidgets.setInterval(1000);
-    timeWaitForXYWidgets.setSingleShot(true);
-
-    connect(&timeWaitForXYWidgets, SIGNAL(timeout()), &xyWidgetsHideTransition, SLOT(start()));
-
-    xyWidgetsShowTransition.setInterval(30);
-    xyWidgetsHideTransition.setInterval(30);
-
-    connect(&xyWidgetsShowTransition, SIGNAL(timeout()), this, SLOT(showXYWidgets()));
-    connect(&xyWidgetsHideTransition, SIGNAL(timeout()), this, SLOT(hideXYWidgets()));
-
-
     repaintTimer.setInterval(100);
     repaintTimer.setSingleShot(true);
     connect(&repaintTimer, SIGNAL(timeout()), this, SLOT(reactivateSmoothing()));
 
-    timerX.setInterval(35);
+    zoomMultiplier = DEFAULT_ZOOM_MULTIPLIER;
+
+    screenRefreshRate = 60;
+
+    timerX.setInterval(1000/screenRefreshRate);
     connect(&timerX, SIGNAL(timeout()), this, SLOT(zoomX()));
 
-    timerY.setInterval(35);
+    timerY.setInterval(1000/screenRefreshRate);
     connect(&timerY, SIGNAL(timeout()), this, SLOT(zoomY()));
 
     dispPoint = false;
     buttonPresse = false;
 
-    sourisSurUneCurve = dispRectangle = recalculate = recalculateRegs = false;
-    hHideStarted = vHideStarted = xyWidgetsState = mouseState.hovering = false;   
+    mouseOnCurve = dispRectangle = recalculate = recalculateRegs = false;
     moving = false;
 
-    kLabel.setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #FFFFFF, stop: 0.3 #D0D0D0 , stop: 0.75 #FFFFFF, stop: 1 #FFFFFF);"
-                         " border-width: 1px; border-color: #D0D0D0; border-style: solid; border-radius: 10;");
+    kLabel = new QLabel();
+    kLabelContainer = new QWidget();
+
+    QHBoxLayout *layout = new QHBoxLayout();
+    layout->setContentsMargins(10, 4, 10, 6);
+    layout->addWidget(kLabel);
+    kLabelContainer->setLayout(layout);
+
+    kLabelContainer->setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #f4f4f4ff, stop: 0.4 #E8E8E8 , stop: 0.6 #E8E8E8, stop: 1 #f4f4f4ff); "
+                          " border-style: none; border-bottom-right-radius: 10; border-bottom-left-radius: 10;");
+
+    kPopupWidget = new PopupWidget(PopupPos::TOP, 0.1, this);
+    kPopupWidget->setWidget(kLabelContainer);
+    kPopupWidget->setState(PopupState::HIDDEN);
 
     customFunctions << "f(x) =" << "g(x) =" << "h(x) =" << "p(x) =" << "r(x) =" << "m(x) =";
     customSequences << "u<sub>n</sub> =" << "v<sub>n</sub> =" << "l<sub>n</sub> =" << "w<sub>n</sub> =" << "q<sub>n</sub> =" << "t<sub>n</sub> =";
 
-    kLabel.setParent(this);
-    kLabel.hide();
-
     savedGraph = NULL;   
 
     setMouseTracking(true);
@@ -206,19 +182,28 @@ void MainGraph::addOtherWidgets()
     verLayout->addWidget(vSlider);
     verLayout->addWidget(unZoom1);
 
-    vWidget = new QWidget(this);
+    QWidget *vWidget = new QWidget(this);
     vWidget->setLayout(verLayout);
 
+    vPopupWidget = new PopupWidget(PopupPos::RIGHT, 0.5, this);
+    vPopupWidget->setWidget(vWidget);  
+
+    // -------- Horizontal zoom slider
+
     QHBoxLayout *horLayout = new QHBoxLayout();
     horLayout->addWidget(unZoom2);
     horLayout->addWidget(hSlider);
     horLayout->addWidget(zoom2);
 
-
-    hWidget = new QWidget(this);
+    QWidget *hWidget = new QWidget(this);
     hWidget->setLayout(horLayout);
 
-    xWidget = new QWidget(this);    
+    hPopupWidget = new PopupWidget(PopupPos::BOTTOM, 0.5, this);
+    hPopupWidget->setWidget(hWidget);
+
+    //-------- x label widget
+
+    QWidget *xWidget = new QWidget(this);
     xWidget->setStyleSheet("border-top-right-radius: 8; border-top-left-radius: 8");
     QHBoxLayout *hbox1 = new QHBoxLayout();
     hbox1->addStretch();
@@ -227,9 +212,14 @@ void MainGraph::addOtherWidgets()
     hbox1->addStretch();
     xWidget->setLayout(hbox1);
     xWidget->setAutoFillBackground(true);
+    xWidget->adjustSize();
 
+    xPopupWidget = new PopupWidget(PopupPos::BOTTOM, 0.25, this);
+    xPopupWidget->setWidget(xWidget);
+    xPopupWidget->setState(PopupState::HIDDEN);
+
+    QWidget *yWidget = new QWidget(this);
 
-    yWidget = new QWidget(this);    
     yWidget->setStyleSheet("border-top-right-radius: 8; border-top-left-radius: 8");
     QHBoxLayout *hbox2 = new QHBoxLayout();
     hbox2->addStretch();
@@ -238,12 +228,17 @@ void MainGraph::addOtherWidgets()
     hbox2->addStretch();
     yWidget->setLayout(hbox2);
     yWidget->setAutoFillBackground(true);
+    yWidget->adjustSize();
 
-    vWidget->setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #FFFFFF, stop: 0.4 #E8E8E8 , stop: 0.6 #E8E8E8, stop: 1 #FFFFFF); "
-                           "border-width: 1px; border-color: #D0D0D0; border-style: solid; border-style: solid; border-top-left-radius: 10; border-bottom-right-radius: 10;");
+    yPopupWidget = new PopupWidget(PopupPos::BOTTOM, 0.75, this);
+    yPopupWidget->setWidget(yWidget);
+    yPopupWidget->setState(PopupState::HIDDEN);
 
-    hWidget->setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #FFFFFF, stop: 0.4 #E8E8E8 , stop: 0.6 #E8E8E8, stop: 1 #FFFFFF); "
-                           "border-width: 1px; border-color: #D0D0D0; border-style: solid; border-top-left-radius: 10; border-bottom-right-radius: 10;");
+    vWidget->setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 #f4f4f4ff, stop: 0.4 #E8E8E8 , stop: 0.6 #E8E8E8, stop: 1 #f4f4f4ff); "
+                           " border-style: none; border-top-left-radius: 16; border-bottom-left-radius: 16;");
+
+    hWidget->setStyleSheet("background-color: QLinearGradient( x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #f4f4f4ff, stop: 0.4 #E8E8E8 , stop: 0.6 #E8E8E8, stop: 1 #f4f4f4ff); "
+                           "border-style: none; border-top-left-radius: 16; border-top-right-radius: 16;");
     hSlider->setStyleSheet("background-color: none; border: none;");
     zoom2->setStyleSheet("background-color: none; border: none;");
     unZoom2->setStyleSheet("background-color: none; border: none;");
@@ -252,80 +247,6 @@ void MainGraph::addOtherWidgets()
     unZoom1->setStyleSheet("background-color: none; border: none;");
 }
 
-void MainGraph::hideHorWidget()
-{   
-    if(hWidget->y() >= hBottom)
-    {
-        hWidgetHideTransition.stop();
-        hWidget->hide();
-        hWidgetState = false;
-        hHideStarted = false;
-    }
-    else hWidget->move(hWidget->x(), hWidget->y()+6);
-}
-
-void MainGraph::hideVerWidget()
-{  
-    if(vWidget->x() >= vBottom)
-    {
-        vWidgetHideTransition.stop();
-        vWidget->hide();
-        vWidgetState = false;
-        vHideStarted = false;
-    }
-    else vWidget->move(vWidget->x()+6, vWidget->y());
-}
-
-void MainGraph::hideXYWidgets()
-{
-    xWidget->move(xWidget->x(), xWidget->y()+6);
-    yWidget->move(yWidget->x(), yWidget->y()+6);
-    if(xWidget->y() >= xyBottom)
-    {
-        xyWidgetsState = false;
-        xyWidgetsHideTransition.stop();
-        xWidget->hide();
-        yWidget->hide();
-    }
-}
-
-void MainGraph::showHorWidget()
-{    
-    hWidget->show();
-
-    if(hWidget->y() <= hTopLeft.y())
-    {
-        hWidgetState = true;
-        hWidgetShowTransition.stop();
-
-    }
-    else hWidget->move(hWidget->x(), hWidget->y()-6);
-}
-
-void MainGraph::showVerWidget()
-{
-    vWidget->show();
-    if(vWidget->x() <= vTopLeft.x())
-    {
-        vWidgetState = true;
-        vWidgetShowTransition.stop();        
-    }
-    else vWidget->move(vWidget->x()-6, vWidget->y());
-}
-
-void MainGraph::showXYWidgets()
-{
-    xWidget->show();
-    yWidget->show();
-    xWidget->move(xWidget->x(), xWidget->y()-6);
-    yWidget->move(yWidget->x(), yWidget->y()-6);
-    if(xWidget->y() <= xTopLeft.y())
-    {
-        xyWidgetsState = true;
-        xyWidgetsShowTransition.stop();
-    }
-}
-
 void MainGraph::lineXReturnPressed()
 {
     bool ok = true;
@@ -345,21 +266,24 @@ void MainGraph::wheelEvent(QWheelEvent *event)
     x = (x-centre.x)/uniteX;
     y = -(y-centre.y)/uniteY;
 
-    double valeur = tanh((double)(event->angleDelta().y()) / 1024) / 1.1;
+    double multiplier = tanh((double)(event->angleDelta().y()) / 1024) / 1.1;
 
-    if((graphRange.Xmax - graphRange.Xmin > MIN_RANGE && graphRange.Ymax - graphRange.Ymin > MIN_RANGE) || valeur < 0)
+    GraphRange win = graphRange;
+
+    win.Xmax -= (win.Xmax - x)*multiplier;
+    win.Xmin -= (win.Xmin - x)*multiplier;
+    win.Ymax -= (win.Ymax - y)*multiplier;
+    win.Ymin -= (win.Ymin - y)*multiplier;
+
+    if( (win.Xmax - win.Xmin > MIN_AMPLITUDE and win.Ymax - win.Ymin > MIN_AMPLITUDE) and
+            (win.Xmax - win.Xmin < MAX_AMPLITUDE and win.Ymax - win.Ymin < MAX_AMPLITUDE))
     {
-        graphRange.Xmax -= (graphRange.Xmax - x)*valeur;
-        graphRange.Xmin -= (graphRange.Xmin - x)*valeur;
-        graphRange.Ymax -= (graphRange.Ymax - y)*valeur;
-        graphRange.Ymin -= (graphRange.Ymin - y)*valeur;
-
         moving = true;
 
         if(parameters.smoothing)
             repaintTimer.start();
 
-        information->setRange(graphRange);
+        information->setRange(win);
     }
 
     event->accept();
@@ -408,47 +332,32 @@ void MainGraph::newWindowSize()
     emit sizeChanged(graphWidth, graphHeight);
     windowSize = size();
 
-    if(hWidget != NULL)
+    vPopupWidget->updatePositions();
+    hPopupWidget->updatePositions();
+    kPopupWidget->updatePositions();
+    xPopupWidget->updatePositions();
+    yPopupWidget->updatePositions();
+
+}
+
+void MainGraph::showEvent(QShowEvent *event)
+{
+    /* Wait for showEvent to determine screen refresh rate
+       to make zooming as smooth as the screen enables.*/
+
+    GraphDraw::showEvent(event);
+
+    QWindow *appWindow = window()->windowHandle();
+    if(appWindow != nullptr)
     {
-        hTopLeft.setX((width()-hWidget->width())/2);
-        hTopLeft.setY(height()-hWidget->height());
-
-        hBottom = hTopLeft.y()+hWidget->height()+5;
-
-        xTopLeft.setX(hTopLeft.x()-xWidget->width());
-        xTopLeft.setY(hTopLeft.y());
-
-        yTopLeft.setX(hTopLeft.x()+hWidget->width());
-        yTopLeft.setY(hTopLeft.y());
-
-        vTopLeft.setX(width()-vWidget->width());
-        vTopLeft.setY((height()-vWidget->height())/2);
-
-        xyBottom = yTopLeft.y()+yWidget->height()+5;
-
-        vBottom = vTopLeft.x()+vWidget->width()+5;
-
-        yWidget->move(yTopLeft);
-        xWidget->move(xTopLeft);
-
-        if(!xyWidgetsState && !selectedCurve.isSomethingSelected)
-            timeWaitForXYWidgets.start();
-
-        vWidget->move(vTopLeft);
-        hWidget->move(hTopLeft);
-
-        hWidgetRect.setTopLeft(hTopLeft);
-        hWidgetRect.setHeight(hWidget->height());
-        hWidgetRect.setWidth(hWidget->width());
-
-        vWidgetRect.setTopLeft(vTopLeft);
-        vWidgetRect.setHeight(vWidget->height());
-        vWidgetRect.setWidth(vWidget->width());
-
-        hWidgetState = true;
-        vWidgetState = true;
-
+        screenRefreshRate = appWindow->screen()->refreshRate();
+        qDebug() << "Screen refresh rate: " << screenRefreshRate;
     }
+    else qDebug() << "Window handle doesn't exist yet at this point";
+
+    timerX.setInterval(1000/screenRefreshRate);
+    timerY.setInterval(1000/screenRefreshRate);
+
 }
 
 void MainGraph::paintEvent(QPaintEvent *event)
@@ -466,11 +375,16 @@ void MainGraph::paintEvent(QPaintEvent *event)
        recalculate = true;       
     }
 
-    if(!moving && (typeCurseur == NORMAL || hWidgetHideTransition.isActive() || vWidgetHideTransition.isActive() ||
-            hWidgetShowTransition.isActive() || vWidgetShowTransition.isActive() || hWidgetState || vWidgetState ||
-                   animationUpdate))
+    if(!moving && (typeCurseur == NORMAL or animationUpdate))
+    {
+        qDebug() << "indirect paint " << qrand();
         indirectPaint();
-    else directPaint();    
+    }
+    else
+    {
+        qDebug() << "direct paint " << qrand();
+        directPaint();
+    }
 
     event->accept();
 }
@@ -487,7 +401,7 @@ void MainGraph::indirectPaint()
 
     painter.setFont(information->getSettingsVals().graphFont);
 
-    painter.drawImage(QPoint(0,0), *savedGraph);    
+    painter.drawImage(QPoint(0,0), *savedGraph);
     painter.translate(QPointF(centre.x, centre.y));
 
     drawAnimatedParEq();
@@ -512,32 +426,31 @@ void MainGraph::directPaint()
     resaveGraph = true;
 
     painter.begin(this);
-    //trace du background
 
+    paintGraph();
+
+    painter.end();
+}
+
+void MainGraph::paintGraph(bool bufferPaint)
+{
     painter.setFont(information->getSettingsVals().graphFont);
 
-    painter.setBrush(QBrush(parameters.backgroundColor));
-    painter.drawRect(-1, -1, graphWidth+1, graphHeight+1);
-
     updateCenterPosAndScaling();
-    drawAxes();
-    drawTicksAndNumbers();
 
     if(updateTickSpacing())
     {
         cancelUpdateSignal = true;
         information->setRange(graphRange);
-
-        painter.setBrush(QBrush(parameters.backgroundColor));
-        painter.drawRect(-1, -1, graphWidth+1, graphHeight+1);
-
-        updateCenterPosAndScaling();
-        drawAxes();
-        drawTicksAndNumbers();
     }
 
+    painter.setBrush(QBrush(parameters.backgroundColor));
+    painter.drawRect(-1, -1, graphWidth+1, graphHeight+1);
 
-    if(dispRectangle)
+    drawAxes();
+    drawTicksAndNumbers();
+
+    if(not bufferPaint and dispRectangle)
     {
         painter.setBrush(Qt::NoBrush);
         pen.setWidth(1);
@@ -564,14 +477,16 @@ void MainGraph::directPaint()
     drawSequences();
     drawStraightLines();
     drawTangents();
-    drawAllParEq();
+
+    if(bufferPaint)
+        drawStaticParEq();
+    else drawAllParEq();
+
     drawRegressions();
     drawData();
 
-    if(dispPoint)
+    if(not bufferPaint and dispPoint)
         drawPoint();
-
-    painter.end();
 }
 
 void MainGraph::drawHoveringConsequence()
@@ -638,37 +553,8 @@ void MainGraph::resaveImageBuffer()
     savedGraph = new QImage(size(), QImage::Format_RGB32);
 
     painter.begin(savedGraph);
-    //trace du background
 
-    painter.setFont(information->getSettingsVals().graphFont);
-
-    painter.setBrush(QBrush(parameters.backgroundColor));
-    painter.drawRect(-1, -1, graphWidth+1, graphHeight+1);
-
-    updateCenterPosAndScaling();
-    drawAxes();
-    drawTicksAndNumbers();
-
-    if(recalculate)
-    {
-        recalculate = false;
-        funcValuesSaver->calculateAll(uniteX, uniteY, graphRange);
-        recalculateRegVals();
-    }
-    else if(recalculateRegs)
-    {
-        recalculateRegs = false;
-        recalculateRegVals();
-    }
-
-    painter.translate(QPointF(centre.x, centre.y));
-
-    drawFunctions();
-    drawSequences();
-    drawStraightLines();
-    drawTangents();
-    drawStaticParEq();
-    drawRegressions();
+    paintGraph(true);
 
     painter.end();
 }
@@ -722,8 +608,9 @@ void MainGraph::checkIfActiveSelectionConflicts()
         if(removeSelection)
         {
             selectedCurve.isSomethingSelected = false;
-            xyWidgetsHideTransition.start();
-            kLabel.hide();
+            xPopupWidget->hideWidget();
+            yPopupWidget->hideWidget();
+            kPopupWidget->hideWidget();
         }
     }
 }
@@ -814,22 +701,6 @@ void MainGraph::drawPoint()
     xTextLabel->setStyleSheet("color: " + parameters.axesColor.name());
     yTextLabel->setStyleSheet("color: " + parameters.axesColor.name());
 
-
-
-    if(selectedCurve.isParametric)
-    {
-        Range kRange;
-        if(selectedCurve.funcType == FUNC_HOVER)
-            kRange = funcs[selectedCurve.id]->getParametricRange();
-        else kRange = seqs[selectedCurve.id]->getKRange();
-
-        kLabel.move(5, 5);
-        double result = kRange.start + selectedCurve.kPos * kRange.step;
-        kLabel.setText("k = " + QString::number(result));
-        kLabel.adjustSize();
-        kLabel.show();
-    }
-
     if(selectedCurve.funcType == FUNC_HOVER)
         pen.setColor(funcs[selectedCurve.id]->getColorSaver()->getColor(selectedCurve.kPos));
     else  pen.setColor(seqs[selectedCurve.id]->getColorSaver()->getColor(selectedCurve.kPos));
@@ -906,8 +777,27 @@ void MainGraph::mouseReleaseEvent(QMouseEvent *event)
                 tangentDrawException = selectedCurve.id;
                 resaveGraph = true;
             }
-            else if(!xyWidgetsState)
-                xyWidgetsShowTransition.start();
+            else
+            {
+                if(selectedCurve.isParametric)
+                {
+                    Range kRange;
+                    if(selectedCurve.funcType == FUNC_HOVER)
+                        kRange = funcs[selectedCurve.id]->getParametricRange();
+                    else kRange = seqs[selectedCurve.id]->getKRange();
+
+                    double result = kRange.start + selectedCurve.kPos * kRange.step;
+                    kLabel->setText("k = " + QString::number(result));
+                    kLabel->adjustSize();
+                    kLabelContainer->adjustSize();
+
+                    kPopupWidget->updatePositions();
+                    kPopupWidget->showWidget();
+                }
+
+                xPopupWidget->showWidget();
+                yPopupWidget->showWidget();
+            }
         }
         else
         {
@@ -917,8 +807,9 @@ void MainGraph::mouseReleaseEvent(QMouseEvent *event)
                     resaveTangent = true;
 
                 selectedCurve.isSomethingSelected = false;
-                xyWidgetsHideTransition.start();
-                kLabel.hide();
+                xPopupWidget->hideWidget();
+                yPopupWidget->hideWidget();
+                kPopupWidget->hideWidget();
 
             }
             else moving = false;            
@@ -939,8 +830,12 @@ void MainGraph::mouseReleaseEvent(QMouseEvent *event)
             win.Ymax = (- rectReel.top() + centre.y) / uniteY;
             win.Ymin = (- rectReel.bottom() + centre.y) / uniteY;
 
-            if(win.Xmax - win.Xmin > MIN_RANGE && win.Ymax - win.Ymin > MIN_RANGE)
+            if( (win.Xmax - win.Xmin > MIN_AMPLITUDE and win.Ymax - win.Ymin > MIN_AMPLITUDE) and
+                    (win.Xmax - win.Xmin < MAX_AMPLITUDE and win.Ymax - win.Ymin < MAX_AMPLITUDE))
+            {
+                resaveGraph = true;
                 information->setRange(win);
+            }
 
             typeCurseur = NORMAL;
         }
@@ -954,29 +849,13 @@ void MainGraph::mouseMoveEvent(QMouseEvent *event)
     mouseX = event->x();
     mouseY = event->y();
 
-    if(hWidgetRect.contains(mouseX, mouseY))
-    {
-        hWidgetHideTransition.stop();
-        hHideStarted = false;
-        hWidgetShowTransition.start();
-    }
-    else if(hWidgetState && !hHideStarted)
-    {
-        mouseNotOnHWidget.start();
-        hHideStarted = true;
-    }
+    if(hPopupWidget->geometry().contains(mouseX, mouseY))
+       hPopupWidget->showWidget();
+    else hPopupWidget->hideWidget();
 
-    if(vWidgetRect.contains(mouseX, mouseY))
-    {        
-        vWidgetHideTransition.stop();
-        vHideStarted = false;
-        vWidgetShowTransition.start();
-    }
-    else if(vWidgetState && !vHideStarted)
-    {
-        mouseNotOnVWidget.start();
-        vHideStarted = true;
-    }
+    if(vPopupWidget->geometry().contains(mouseX, mouseY))
+       vPopupWidget->showWidget();
+    else vPopupWidget->hideWidget();
 
     if(typeCurseur == NORMAL)
     {
@@ -1228,10 +1107,6 @@ void MainGraph::mouseSeqHoverTest(double x, double y)
             }
         }
     }
-
-
-
-
 }
 
 void MainGraph::mouseTangentHoverTest(double x, double y)
@@ -1295,41 +1170,49 @@ void MainGraph::drawTicksAndNumbers()
     double start, end, Ypos, posTxt;
 
     //trace sur l'axe des X
-    if(centre.y < 20)
-    {       
-        Ypos = 20;
-        posTxt = Ypos + parameters.graphFont.pixelSize() + 3;
-    }
-    else if(graphHeight - centre.y < 20)
-    {       
-        Ypos = graphHeight - 20;
-        posTxt = Ypos - 7;
-    }
-    else
-    {
-        Ypos = centre.y;
-        posTxt = Ypos + parameters.graphFont.pixelSize() + 3;
-    }
 
-    double Xreal = trunc(graphRange.Xmin / graphRange.Xscale) * graphRange.Xscale;
+
+    double Xreal = trunc(graphRange.Xmin / graphRange.Xstep) * graphRange.Xstep;
     double Xpos = Xreal * uniteX + centre.x;
     double pos;
 
-    double step = graphRange.Xscale * uniteX;
+    double step = graphRange.Xstep * uniteX;
 
     double bas = height();
     double haut = 0;
 
     QString num = QString::number(Xreal, 'g', NUM_PREC);
-    widestXNumber = painter.fontMetrics().width(num);
+    int num_width = painter.fontMetrics().boundingRect(num).width();
+    int num_height = painter.fontMetrics().boundingRect(num).height();
+
+    auto update_postTxt_y = [&]()
+    {
+        if(centre.y < 20)
+        {
+            Ypos = 20;
+            posTxt = Ypos + num_height + 3;
+        }
+        else if(graphHeight - centre.y < 20)
+        {
+            Ypos = graphHeight - 20;
+            posTxt = Ypos - 7;
+        }
+        else
+        {
+            Ypos = centre.y;
+            posTxt = Ypos + num_height + 3;
+        }
+    };
+
+    widestXNumber = num_width;
 
     start = 5;
     end = graphWidth - 5;
 
     if(centre.x < 10)
-        start = 10 + painter.fontMetrics().width(num)/2 + 5;
+        start = 10 + num_width/2 + 5;
     else if(centre.x > graphWidth - 10)
-        end = graphWidth - 10 - painter.fontMetrics().width(num)/2 - 5;
+        end = graphWidth - 10 - num_width/2 - 5;
 
     while(Xpos <= end)
     {       
@@ -1349,15 +1232,19 @@ void MainGraph::drawTicksAndNumbers()
 
             painter.drawLine(QPointF(Xpos, Ypos -3), QPointF(Xpos, Ypos));
             num = QString::number(Xreal, 'g', NUM_PREC);
-            pos = Xpos - painter.fontMetrics().width(num)/2;
+            num_width = painter.fontMetrics().boundingRect(num).width();
+            num_height = painter.fontMetrics().boundingRect(num).height();
+            update_postTxt_y();
+
+            pos = Xpos - num_width/2;
             painter.drawText(QPointF(pos, posTxt), num);
 
-            if(painter.fontMetrics().width(num) > widestXNumber)
-                widestXNumber = painter.fontMetrics().width(num);
+            if(num_width > widestXNumber)
+                widestXNumber = num_width;
         }
 
         Xpos += step;
-        Xreal += graphRange.Xscale;
+        Xreal += graphRange.Xstep;
     }
 
 //trace sur l'axe des Y
@@ -1381,9 +1268,9 @@ void MainGraph::drawTicksAndNumbers()
         posTxt = Xpos + 5;
     }
 
-    double Yreal = trunc(graphRange.Ymax / graphRange.Yscale) * graphRange.Yscale;
+    double Yreal = trunc(graphRange.Ymax / graphRange.Ystep) * graphRange.Ystep;
     Ypos = -Yreal * uniteY + centre.y;
-    step = graphRange.Yscale * uniteY;
+    step = graphRange.Ystep * uniteY;
 
     bas =  0;
     haut =  graphWidth;
@@ -1418,49 +1305,86 @@ void MainGraph::drawTicksAndNumbers()
 
             painter.drawLine(QPointF(Xpos  -3, Ypos), QPointF(Xpos, Ypos));
             num = QString::number(Yreal, 'g', NUM_PREC);
+            num_width = painter.fontMetrics().boundingRect(num).width();
             if(drawOnRight)
-                painter.drawText(QPointF(posTxt - painter.fontMetrics().width(num), Ypos + txtCorr), num);
+                painter.drawText(QPointF(posTxt - num_width, Ypos + txtCorr), num);
             else painter.drawText(QPointF(posTxt, Ypos + txtCorr), num);
         }
 
-        Yreal -= graphRange.Yscale;
+        Yreal -= graphRange.Ystep;
         Ypos += step;
     }   
 }
 
+void MainGraph::incrementTickSpacing(double &spacing, int &currentMultiplier)
+{
+    switch (currentMultiplier) {
+    case 1:
+        currentMultiplier = 2;
+        spacing *= 2;
+        break;
+    case 2:
+        currentMultiplier = 5;
+        spacing *= 2.5;
+        break;
+    case 5:
+        currentMultiplier = 1;
+        spacing *= 2;
+        break;
+    }
+}
+
+void MainGraph::decrementTickSpacing(double &spacing, int &currentMultiplier)
+{
+    switch (currentMultiplier) {
+    case 1:
+        currentMultiplier = 5;
+        spacing /= 2;
+        break;
+    case 5:
+        currentMultiplier = 2;
+        spacing /= 2.5;
+        break;
+    case 2:
+        currentMultiplier = 1;
+        spacing /= 2;
+        break;
+    }
+}
+
 bool MainGraph::updateTickSpacing()
 {
     bool scaleChanged = false;
     bool orthonormal = information->isOrthonormal();
 
-    if(uniteX * graphRange.Xscale < widestXNumber + 32)
+    if(uniteX * graphRange.Xstep < widestXNumber + 32)
     {
-        while(uniteX * graphRange.Xscale < widestXNumber + 32)
-            graphRange.Xscale *= 2;
+        while(uniteX * graphRange.Xstep < widestXNumber + 32)
+            incrementTickSpacing(graphRange.Xstep, graphRange.XstepMult);
         if(orthonormal)
-             graphRange.Yscale = graphRange.Xscale;
+             graphRange.Ystep = graphRange.Xstep;
         scaleChanged = true;
     }
-    else if(uniteX * graphRange.Xscale > 2*widestXNumber + 96)
+    else if(uniteX * graphRange.Xstep > 2*widestXNumber + 96)
     {
-        while(uniteX * graphRange.Xscale > 2*widestXNumber + 96)
-            graphRange.Xscale /= 2;
+        while(uniteX * graphRange.Xstep > 2*widestXNumber + 96)
+            decrementTickSpacing(graphRange.Xstep, graphRange.XstepMult);
         if(orthonormal)
-             graphRange.Yscale = graphRange.Xscale;
+             graphRange.Ystep = graphRange.Xstep;
         scaleChanged = true;
     }
     if(!orthonormal)
     {
-        if(uniteY * graphRange.Yscale < 25)
+        if(uniteY * graphRange.Ystep < 25)
         {
-            while(uniteY * graphRange.Yscale < 25)
-                graphRange.Yscale *= 2;
+            while(uniteY * graphRange.Ystep < 25)
+                incrementTickSpacing(graphRange.Ystep, graphRange.YstepMult);
             scaleChanged = true;
         }
-        else if(uniteY * graphRange.Yscale > 150)
+        else if(uniteY * graphRange.Ystep > 150)
         {
-            while(uniteY * graphRange.Yscale > 150)
-                graphRange.Yscale /= 2;
+            while(uniteY * graphRange.Ystep > 150)
+                decrementTickSpacing(graphRange.Ystep, graphRange.YstepMult);
             scaleChanged = true;
         }
     }
@@ -1523,15 +1447,23 @@ void MainGraph::zoomX()
 {
     repaintTimer.stop();
 
-    double valeur = (graphRange.Xmax- graphRange.Xmin) * (double)(hSlider->value()) * 0.0016;
+    GraphRange win = graphRange;
 
-    graphRange.Xmin += valeur;
-    graphRange.Xmax -= valeur;
-    moving = true;
-    information->setRange(graphRange);
+    double valeur = (graphRange.Xmax- graphRange.Xmin) * (double)(hSlider->value()) * zoomMultiplier / screenRefreshRate;
 
-    if(parameters.smoothing)
-        repaintTimer.start();
+    win.Xmin += valeur;
+    win.Xmax -= valeur;
+
+    if( (win.Xmax - win.Xmin > MIN_AMPLITUDE and win.Ymax - win.Ymin > MIN_AMPLITUDE) and
+            (win.Xmax - win.Xmin < MAX_AMPLITUDE and win.Ymax - win.Ymin < MAX_AMPLITUDE))
+    {
+        moving = true;
+
+        if(parameters.smoothing)
+            repaintTimer.start();
+
+        information->setRange(win);
+    }
 }
 
 void MainGraph::stop_X_zoom()
@@ -1544,26 +1476,33 @@ void MainGraph::zoomY()
 {
     repaintTimer.stop();
 
-    double valeur = (graphRange.Ymax - graphRange.Ymin) * (double)(vSlider->value()) * 0.0016;
+    double valeur = (graphRange.Ymax - graphRange.Ymin) * (double)(vSlider->value()) * zoomMultiplier / screenRefreshRate;
+
+    GraphRange win = graphRange;
 
     if(!information->isOrthonormal())
     {        
-        graphRange.Ymin += valeur;
-        graphRange.Ymax -= valeur;
+        win.Ymin += valeur;
+        win.Ymax -= valeur;
         recalculate = false;
     }
     else
     {
-        graphRange.Xmin += valeur;
-        graphRange.Xmax -= valeur;
+        win.Xmin += valeur;
+        win.Xmax -= valeur;
         recalculate = true;
     }
 
-    moving = true;
-    if(parameters.smoothing)
-        repaintTimer.start();
+    if( (win.Xmax - win.Xmin > MIN_AMPLITUDE and win.Ymax - win.Ymin > MIN_AMPLITUDE) and
+            (win.Xmax - win.Xmin < MAX_AMPLITUDE and win.Ymax - win.Ymin < MAX_AMPLITUDE))
+    {
+        moving = true;
 
-    information->setRange(graphRange);
+        if(parameters.smoothing)
+            repaintTimer.start();
+
+        information->setRange(win);
+    }
 }
 
 void MainGraph::stop_Y_Zoom()
